/*
 * n01_stats
 *
 * ListView.c
 *
 * Copyright (C) 1996-2006 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef	_INC_OLE
#include <commctrl.h>
#include <tchar.h>
#include <stdio.h>

#include "resource.h"

#include "general.h"
#include "Message.h"
#include "ImageList.h"
#include "ListView.h"
#include "data.h"

/* Define */
#ifndef LVS_EX_INFOTIP
#define LVS_EX_INFOTIP			0x400
#endif
#ifndef LVM_SETSELECTEDCOLUMN
#define LVM_SETSELECTEDCOLUMN	(LVM_FIRST + 140)
#endif
#ifndef ListView_SetSelectedColumn
#define ListView_SetSelectedColumn(hwnd, iCol) \
			SNDMSG((hwnd), LVM_SETSELECTEDCOLUMN, (WPARAM)iCol, 0)
#endif
#ifndef HDF_SORTDOWN
#define HDF_SORTDOWN			0x0200
#endif
#ifndef HDF_SORTUP
#define HDF_SORTUP				0x0400
#endif

#define ABS(n)					((n < 0) ? (n * -1) : n)

/* Global Variables */
static HEADER_INFO show_hi[MAX_HEADER];
static int show_hi_count;

typedef struct _SORT_INFO {
	int colum;
	int order;
} SORT_INFO;

extern OPTION op;
extern DATA_INFO *g_di;
extern DATA_INFO *g_show_di;

/* Local Function Prototypes */

/*
 * listview_create - Xgr[̍쐬
 */
HWND listview_create(const HINSTANCE hInstance, const HWND hWnd, const int id, const HIMAGELIST icon_list)
{
	HWND hListView;

	// Xgr[̍쐬
	hListView = CreateWindowEx(WS_EX_NOPARENTNOTIFY | WS_EX_CLIENTEDGE,
		WC_LISTVIEW, NULL, WS_VISIBLE | WS_TABSTOP | WS_CHILD |
		LVS_SHAREIMAGELISTS | op.lv_style,
		0, 0, 500, 500, hWnd, (HMENU)id, hInstance, NULL);
	if (hListView == NULL) {
		return NULL;
	}
	ListView_SetExtendedListViewStyle(hListView, op.lv_ex_style);

	// C[WXg̐ݒ
	ListView_SetImageList(hListView, icon_list, LVSIL_SMALL);
	return hListView;
}

/*
 * listview_close - Xgr[̉
 */
void listview_close(const HWND hListView)
{
	return;
}

/*
 * listview_notify_proc - Xgr[bZ[W
 */
LRESULT listview_notify_proc(const HWND hWnd, const HWND hListView, const LPARAM lParam)
{
	NMLVDISPINFO *plv = (NMLVDISPINFO *)lParam;
	NMLVKEYDOWN *lvk = (NMLVKEYDOWN *)lParam;
	NMHDR *CForm = (NMHDR *)lParam;

	switch (plv->hdr.code) {
	case LVN_GETDISPINFO:		// \ACe̗v
	case LVN_ITEMCHANGED:		// ACȇIԂ̕ύX
	case LVN_ITEMACTIVATE:
		return SendMessage(hWnd, WM_LV_EVENT, plv->hdr.code, lParam);
	}

	switch (lvk->hdr.code) {
	case LVN_KEYDOWN:			// L[_E
		return SendMessage(hWnd, WM_LV_EVENT, lvk->hdr.code, lParam);
	}

	switch (CForm->code) {
	case NM_DBLCLK:				// _uNbN
	case NM_CLICK:				// NbN
	case NM_RCLICK:				// ENbN
	case NM_SETFOCUS:
		return SendMessage(hWnd, WM_LV_EVENT, CForm->code, lParam);
	}
	return 0;
}

/*
 * listview_header_notify_proc - Xgr[wb_bZ[W
 */
LRESULT listview_header_notify_proc(const HWND hWnd, const HWND hListView, const LPARAM lParam)
{
	NMHEADER *hdn = (NMHEADER *)lParam;
	HDITEM hdi;
	int i;

	switch (hdn->hdr.code) {
	case HDN_ITEMCLICK:
		// \[g
		hdi.mask = HDI_FORMAT;
		Header_GetItem(hdn->hdr.hwndFrom, op.sort_colum, &hdi);
		hdi.fmt &= hdi.fmt ^ (HDF_SORTUP | HDF_SORTDOWN);
		Header_SetItem(hdn->hdr.hwndFrom, op.sort_colum, &hdi);

		op.sort_order = (op.sort_colum == hdn->iItem) ? !op.sort_order : 0;
		op.sort_colum = hdn->iItem;
		listview_sort(hListView, op.sort_colum, op.sort_order);
		break;

	case HDN_ITEMCHANGED:
		if (!(hdn->pitem->mask & HDI_WIDTH)) {
			break;
		}
		for (i = 0; i < op.hi_count; i++) {
			if (op.hi[i].type == show_hi[hdn->iItem].type) {
				if (hdn->pitem->cxy > 0) {
					op.hi[i].cx = hdn->pitem->cxy;
				}
				break;
			}
		}
		break;
	}
	return 0;
}

/*
 * listview_sort - Xgr[̃\[g
 */
void listview_sort(const HWND hListView, int colum, int order)
{
	SORT_INFO si;
	HDITEM hdi;

	si.colum = colum;
	si.order = order;
	ListView_SortItems(hListView, listview_compare_func, &si);
	ListView_SetSelectedColumn(hListView, colum);

	hdi.mask = HDI_FORMAT;
	Header_GetItem(ListView_GetHeader(hListView), colum, &hdi);
	hdi.fmt &= hdi.fmt ^ (HDF_SORTUP | HDF_SORTDOWN);
	hdi.fmt |= (order == 0) ? HDF_SORTUP : HDF_SORTDOWN;
	Header_SetItem(ListView_GetHeader(hListView), colum, &hdi);
}

/*
 * listview_set_column - J̐ݒ
 */
static int __cdecl compare(const void *arg1, const void *arg2)
{
   return (((HEADER_INFO *)arg1)->order - ((HEADER_INFO *)arg2)->order);
}
BOOL listview_set_column(const HWND hListView, HEADER_INFO *hi, const int hi_count)
{
	LV_COLUMN lvc;
	int i;

	// J̍폜
	while (Header_GetItemCount(ListView_GetHeader(hListView)) > 0) {
		ListView_DeleteColumn(hListView, 0);
	}
	// \J̐ݒ
	show_hi_count = 0;
	for (i = 0; i < hi_count; i++) {
		if (hi[i].show == 0) {
			continue;
		}
		show_hi[show_hi_count++] = hi[i];
	}
	qsort(&show_hi, show_hi_count, sizeof(HEADER_INFO), compare);

	// Jݒ
	lvc.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
	for (i = 0; i < show_hi_count; i++) {
		lvc.fmt = show_hi[i].fmt;
		lvc.cx = show_hi[i].cx;
		lvc.pszText = data_get_title(show_hi[i].type);
		lvc.iSubItem = i;
		ListView_InsertColumn(hListView, lvc.iSubItem, &lvc);
	}
	return TRUE;
}

/*
 * listview_set_lparam - ACeLPARAMݒ
 */
BOOL listview_set_lparam(const HWND hListView, const int i, const LPARAM lParam)
{
	LV_ITEM lvi;

	ZeroMemory(&lvi, sizeof(LVITEM));
	lvi.mask = LVIF_PARAM;
	lvi.iItem = i;
	lvi.lParam = lParam;
	return ListView_SetItem(hListView, &lvi);
}

/*
 * listview_get_lparam - ACeLPARAM擾
 */
LPARAM listview_get_lparam(const HWND hListView, const int i)
{
	LV_ITEM lvi;

	ZeroMemory(&lvi, sizeof(LVITEM));
	lvi.mask = LVIF_PARAM;
	lvi.iItem = i;
	ListView_GetItem(hListView, &lvi);
	return lvi.lParam;
}

/*
 * listview_set_item - ACe̒ǉ
 */
int listview_set_item(const HWND hListView, const int i, const LPARAM lParam)
{
	LV_ITEM lvi;

	lvi.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM;
	lvi.iItem = (i < 0) ? ListView_GetItemCount(hListView) : i;
	lvi.iSubItem = 0;
	lvi.pszText = LPSTR_TEXTCALLBACK;
	lvi.cchTextMax = 0;
	lvi.iImage = I_IMAGECALLBACK;
	lvi.lParam = lParam;
	return ListView_InsertItem(hListView, &lvi);
}

/*
 * ListView_GetDispItem - Xgr[ɕ\ACe̐ݒ
 */
BOOL listview_get_disp_item(const HWND hListView, LV_DISPINFO *lv_disp)
{
	DATA_INFO *di;

	di = &g_show_di[lv_disp->item.lParam];
	if (di == NULL) {
		return FALSE;
	}
	// ACR
	if (lv_disp->item.mask & LVIF_IMAGE) {
		if (di->index >= 0) {
			lv_disp->item.iImage = ICON_DATA;
		} else {
			lv_disp->item.iImage = ICON_DATA_M;
		}
	}
	// eLXg
	if (lv_disp->item.mask & LVIF_TEXT) {
		data_to_value(show_hi[lv_disp->item.iSubItem].type, di, lv_disp->item.pszText);
	}
	return TRUE;
}

/*
 * listview_compare_func - \[gpr
 */
int CALLBACK listview_compare_func(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	DATA_INFO *di1 = &g_show_di[lParam1];
	DATA_INFO *di2 = &g_show_di[lParam2];
	SORT_INFO *si = (SORT_INFO *)lParamSort;
	TCHAR str1[BUF_SIZE], str2[BUF_SIZE];
	TCHAR *p1, *p2;
	FILETIME ft1, ft2;
	float f1, f2;
	int ret = 0;

	switch (show_hi[si->colum].type) {
	case HDTP_NAME:
		ret = lstrcmpi(di1->name, di2->name);
		break;
	case HDTP_LINK_NAME:
		if (di1->index >= 0 && g_di[di1->index].index >= 0) {
			p1 = g_di[g_di[di1->index].index].name;
		} else {
			p1 = TEXT("");
		}
		if (di2->index >= 0 && g_di[di2->index].index >= 0) {
			p2 = g_di[g_di[di2->index].index].name;
		} else {
			p2 = TEXT("");
		}
		ret = lstrcmpi(p1, p2);
		break;
	case HDTP_VS_NAME:
		if (di1->vs_index >= 0) {
			p1 = g_di[di1->vs_index].name;
		} else {
			p1 = TEXT("");
		}
		if (di2->vs_index >= 0) {
			p2 = g_di[di2->vs_index].name;
		} else {
			p2 = TEXT("");
		}
		ret = lstrcmpi(p1, p2);
		break;
	case HDTP_START_SCORE:
		ret = di1->start_score - di2->start_score;
		break;
	case HDTP_DATE:
		if (di1->start_time.wYear <= 0 || di2->start_time.wYear <= 0 ||
			di1->start_time.wMonth <= 0 || di2->start_time.wMonth <= 0 ||
			di1->start_time.wDay <= 0 || di2->start_time.wDay <= 0) {
			wsprintf(str1, TEXT("%04d%02d%02d"), di1->start_time.wYear, di1->start_time.wMonth, di1->start_time.wDay);
			wsprintf(str2, TEXT("%04d%02d%02d"), di2->start_time.wYear, di2->start_time.wMonth, di2->start_time.wDay);
			ret = lstrcmpi(str1, str2);
		} else {
			SystemTimeToFileTime(&di1->start_time, &ft1);
			SystemTimeToFileTime(&di2->start_time, &ft2);
			ret = CompareFileTime(&ft1, &ft2);
		}
		break;
	case HDTP_FILE_NAME:
		ret = lstrcmpi(di1->file_name, di2->file_name);
		break;
	case HDTP_SETS_PLAYED:
		ret = di1->sets_play - di2->sets_play;
		break;
	case HDTP_SETS_WON:
		ret = di1->sets - di2->sets;
		break;
	case HDTP_LEGS_PLAYED:
		ret = di1->legs_play - di2->legs_play;
		break;
	case HDTP_LEGS_WON:
		ret = di1->legs - di2->legs;
		break;
	case HDTP_TON:
		ret = di1->ton_count - di2->ton_count;
		break;
	case HDTP_TON00:
		ret = di1->ton00_count - di2->ton00_count;
		break;
	case HDTP_TON40:
		ret = di1->ton40_count - di2->ton40_count;
		break;
	case HDTP_TON80:
		ret = di1->ton80_count - di2->ton80_count;
		break;
	case HDTP_HIGH_OFF:
		ret = di1->high_off - di2->high_off;
		break;
	case HDTP_SHORT_GAME:
		ret = di1->short_game - di2->short_game;
		break;
	case HDTP_LONG_GAME:
		ret = di1->long_game - di2->long_game;
		break;
	case HDTP_AVG_SET:
		f1 = (di1->sets_play <= 0) ? 0 : ((float)(di1->sets) / (float)(di1->sets_play)) * 100;
		f2 = (di2->sets_play <= 0) ? 0 : ((float)(di2->sets) / (float)(di2->sets_play)) * 100;
		ret = (f1 == f2) ? 0 : ((f1 < f2) ? -1 : 1);
		break;
	case HDTP_AVG_LEG:
		f1 = (di1->legs_play <= 0) ? 0 : ((float)(di1->legs) / (float)(di1->legs_play)) * 100;
		f2 = (di2->legs_play <= 0) ? 0 : ((float)(di2->legs) / (float)(di2->legs_play)) * 100;
		ret = (f1 == f2) ? 0 : ((f1 < f2) ? -1 : 1);
		break;
	case HDTP_AVG_DARTS:
		f1 = (di1->win_count <= 0) ? 0 : ((float)(di1->win_darts) / (float)(di1->win_count)) * 100;
		f2 = (di2->win_count <= 0) ? 0 : ((float)(di2->win_darts) / (float)(di2->win_count)) * 100;
		ret = (f1 == f2) ? 0 : ((f1 < f2) ? -1 : 1);
		break;
	case HDTP_AVG_SCORE:
		f1 = (di1->all_darts <= 0) ? 0 : ((float)(di1->all_score) / (float)(di1->all_darts)) * 100;
		f2 = (di2->all_darts <= 0) ? 0 : ((float)(di2->all_score) / (float)(di2->all_darts)) * 100;
		ret = (f1 == f2) ? 0 : ((f1 < f2) ? -1 : 1);
		break;
	case HDTP_AVG_FIRST9:
		f1 = (di1->first9_darts <= 0) ? 0 : ((float)(di1->first9_score) / (float)(di1->first9_darts)) * 100;
		f2 = (di2->first9_darts <= 0) ? 0 : ((float)(di2->first9_score) / (float)(di2->first9_darts)) * 100;
		ret = (f1 == f2) ? 0 : ((f1 < f2) ? -1 : 1);
		break;
	case HDTP_AVG_CHECK_OUT:
		f1 = (di1->check_out_aim <= 0) ? 0 : ((float)(di1->check_out) / (float)(di1->check_out_aim)) * 100;
		f2 = (di2->check_out_aim <= 0) ? 0 : ((float)(di2->check_out) / (float)(di2->check_out_aim)) * 100;
		ret = (f1 == f2) ? 0 : ((f1 < f2) ? -1 : 1);
		break;
	case HDTP_AVG_CHECK_OUT_COUNT:
		wsprintf(str1, TEXT("(%06d / %06d)"), di1->check_out, di1->check_out_aim);
		wsprintf(str2, TEXT("(%06d / %06d)"), di2->check_out, di2->check_out_aim);
		ret = lstrcmpi(str1, str2);
		break;
	case HDTP_AVG_KEEP_LEGS:
		f1 = (di1->all_keep_legs <= 0) ? 0 : ((float)(di1->win_keep_legs) / (float)(di1->all_keep_legs)) * 100;
		f2 = (di2->all_keep_legs <= 0) ? 0 : ((float)(di2->win_keep_legs) / (float)(di2->all_keep_legs)) * 100;
		ret = (f1 == f2) ? 0 : ((f1 < f2) ? -1 : 1);
		break;
	case HDTP_AVG_KEEP_LEGS_COUNT:
		wsprintf(str1, TEXT("(%06d / %06d)"), di1->win_keep_legs, di1->all_keep_legs);
		wsprintf(str2, TEXT("(%06d / %06d)"), di2->win_keep_legs, di2->all_keep_legs);
		ret = lstrcmpi(str1, str2);
		break;
	case HDTP_AVG_BREAK_LEGS:
		f1 = (di1->all_break_legs <= 0) ? 0 : ((float)(di1->win_break_legs) / (float)(di1->all_break_legs)) * 100;
		f2 = (di2->all_break_legs <= 0) ? 0 : ((float)(di2->win_break_legs) / (float)(di2->all_break_legs)) * 100;
		ret = (f1 == f2) ? 0 : ((f1 < f2) ? -1 : 1);
		break;
	case HDTP_AVG_BREAK_LEGS_COUNT:
		wsprintf(str1, TEXT("(%06d / %06d)"), di1->win_break_legs, di1->all_break_legs);
		wsprintf(str2, TEXT("(%06d / %06d)"), di2->win_break_legs, di2->all_break_legs);
		ret = lstrcmpi(str1, str2);
		break;
	case HDTP_SUCCESS_2_80:
		ret = di1->success_2_80 - di2->success_2_80;
		break;
	case HDTP_FAILURE_2_80:
		ret = di1->failure_2_80 - di2->failure_2_80;
		break;
	case HDTP_SUCCESS_81_130:
		ret = di1->success_81_130 - di2->success_81_130;
		break;
	case HDTP_FAILURE_81_130:
		ret = di1->failure_81_130 - di2->failure_81_130;
		break;
	case HDTP_SUCCESS_131:
		ret = di1->success_131 - di2->success_131;
		break;
	case HDTP_FAILURE_131:
		ret = di1->failure_131 - di2->failure_131;
		break;
	}
	return (((ret < 0 && si->order == 1) || (ret > 0 && si->order == 0)) ? 1 : -1);
}
/* End of source */
