/*
 * n01_stats
 *
 * TreeView.c
 *
 * Copyright (C) 1996-2006 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef	_INC_OLE
#include <commctrl.h>

#include "General.h"
#include "Memory.h"
#include "String.h"
#include "Message.h"
#include "ImageList.h"
#include "TreeView.h"

#include "resource.h"

/* Define */

/* Global Variables */

/* Local Function Prototypes */

/*
 * treeview_create - c[r[̍쐬
 */
HWND treeview_create(const HINSTANCE hInstance, const HWND hWnd, const int id, const HIMAGELIST icon_list)
{
	HWND hTreeView;

	// c[r[̍쐬
	hTreeView = CreateWindowEx(WS_EX_NOPARENTNOTIFY | WS_EX_CLIENTEDGE,
		WC_TREEVIEW, NULL, WS_VISIBLE | WS_CHILD | WS_TABSTOP |
		TVS_HASLINES | TVS_SHOWSELALWAYS | TVS_HASBUTTONS | TVS_LINESATROOT,
		0, 0, 300, 100, hWnd, (HMENU)id, hInstance, NULL);
	if (hTreeView == NULL) {
		return NULL;
	}
	// C[WXg̐ݒ
	TreeView_SetImageList(hTreeView, icon_list, TVSIL_NORMAL);
	return hTreeView;
}

/*
 * treeview_close - c[r[̉
 */
void treeview_close(const HWND hTreeView)
{
	return;
}

/*
 * treeview_notify_proc - c[r[Cxg
 */
LRESULT treeview_notify_proc(const HWND hWnd, LPARAM lParam)
{
	TV_DISPINFO *tvd = (TV_DISPINFO *)lParam;
	NM_TREEVIEW *nmtv = (NM_TREEVIEW *)lParam;
	TV_KEYDOWN *tvk = (TV_KEYDOWN *)lParam;
	NMHDR *nmhdr = (NMHDR *)lParam;

	switch (tvd->hdr.code) {
	case TVN_BEGINLABELEDIT:
	case TVN_ENDLABELEDIT:
		return SendMessage(hWnd, WM_TV_EVENT, tvd->hdr.code, lParam);
	}

	switch (nmtv->hdr.code) {
	case TVN_BEGINDRAG:
	case TVN_BEGINRDRAG:
	case TVN_SELCHANGING:
	case TVN_SELCHANGED:
	case NM_CUSTOMDRAW:
		return SendMessage(hWnd, WM_TV_EVENT, nmtv->hdr.code, lParam);
	}

	switch (tvk->hdr.code) {
	case TVN_KEYDOWN:			// L[_E
		return SendMessage(hWnd, WM_TV_EVENT, tvk->hdr.code, lParam);
	}

	switch (nmhdr->code) {
	case NM_SETFOCUS:
	case NM_RCLICK:
	case NM_DBLCLK:
		return SendMessage(hWnd, WM_TV_EVENT, nmhdr->code, lParam);
	}
	return FALSE;
}

/*
 * treeview_scroll - c[r[XN[ăACe\
 */
void treeview_scroll(const HWND hTreeView)
{
	RECT rect, item_rect;
	POINT pt;

	// c[r[̃TCY擾
	GetClientRect(hTreeView, &rect);
	// }EXʒu擾
	GetCursorPos(&pt);
	ScreenToClient(hTreeView, &pt);
	//ACe̍擾
	TreeView_GetItemRect(hTreeView, treeview_get_hitest(hTreeView), &item_rect, FALSE);

	// XN[
	if (pt.y < rect.top + (item_rect.bottom - item_rect.top)) {
		SendMessage(hTreeView, WM_VSCROLL, SB_LINEUP, 0);
	} else if (pt.y > rect.bottom - (item_rect.bottom - item_rect.top)) {
		SendMessage(hTreeView, WM_VSCROLL, SB_LINEDOWN, 0);
	}
}

/*
 * treeview_set_item - c[r[ACe̒ǉ
 */
HTREEITEM treeview_set_item(const HWND hTreeView, TCHAR *buf, const HTREEITEM hParent, const HTREEITEM After,
							const int icon, const int select_icon, LPARAM lParam)
{
	TV_INSERTSTRUCT tvitn;
	TV_ITEM tvit;

	tvit.mask = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_PARAM;
	tvit.iImage = icon;
	tvit.iSelectedImage = select_icon;
	tvit.hItem = NULL;
	tvit.state = 0;
	tvit.stateMask = 0;
	tvit.cChildren = 0;
	tvit.lParam = lParam;
	tvit.pszText = buf;
	tvit.cchTextMax = BUF_SIZE - 1;

	tvitn.hInsertAfter = After;
	tvitn.hParent = hParent;
	tvitn.item = tvit;
	return TreeView_InsertItem(hTreeView, &tvitn);
}

/*
 * treeview_delete_child - qACe폜
 */
void treeview_delete_child(const HWND hTreeView, const HTREEITEM parent_item)
{
	HTREEITEM hItem;

	hItem = TreeView_GetNextItem(hTreeView, parent_item, TVGN_CHILD);
	while (hItem != NULL) {
		TreeView_DeleteItem(hTreeView, hItem);
		hItem = TreeView_GetNextItem(hTreeView, parent_item, TVGN_CHILD);
	}
}

/*
 * treeview_set_text - ACẽeLXgݒ
 */
BOOL treeview_set_text(const HWND hTreeView, const HTREEITEM hItem, TCHAR *text)
{
	TV_ITEM tvit;

	if (hItem == NULL) {
		return FALSE;
	}
	tvit.mask = TVIF_TEXT;
	tvit.hItem = hItem;
	tvit.pszText = text;
	tvit.cchTextMax = BUF_SIZE - 1;
	return TreeView_SetItem(hTreeView, &tvit);
}

/*
 * treeview_get_text - ACẽeLXg擾
 */
BOOL treeview_get_text(const HWND hTreeView, const HTREEITEM hItem, TCHAR *text)
{
	TV_ITEM tvit;

	if (hItem == NULL) {
		return FALSE;
	}
	tvit.mask = TVIF_TEXT;
	tvit.hItem = hItem;
	tvit.pszText = text;
	tvit.cchTextMax = BUF_SIZE - 1;
	return TreeView_GetItem(hTreeView, &tvit);
}

/*
 * treeview_set_icon - ACẽACRݒ
 */
BOOL treeview_set_icon(const HWND hTreeView, const HTREEITEM hItem, const int icon, const int select_icon)
{
	TV_ITEM tvit;

	if (hItem == NULL) {
		return FALSE;
	}
	tvit.mask = TVIF_IMAGE | TVIF_SELECTEDIMAGE;
	tvit.hItem = hItem;
	tvit.iImage = icon;
	tvit.iSelectedImage = select_icon;
	return TreeView_SetItem(hTreeView, &tvit);
}

/*
 * treeview_get_icon - ACẽACR擾
 */
int treeview_get_icon(const HWND hTreeView, const HTREEITEM hItem)
{
	TV_ITEM tvit;

	if (hItem == NULL) {
		return FALSE;
	}
	tvit.mask = TVIF_IMAGE;
	tvit.hItem = hItem;
	tvit.iImage = 0;
	TreeView_GetItem(hTreeView, &tvit);
	return tvit.iImage;
}

/*
 * treeview_set_lparam - ACeɏ֘A
 */
BOOL treeview_set_lparam(const HWND hTreeView, const HTREEITEM hItem, const LPARAM lParam)
{
	TV_ITEM tvit;

	if (hItem == NULL) {
		return FALSE;
	}
	tvit.mask = TVIF_PARAM;
	tvit.hItem = hItem;
	tvit.lParam = (LPARAM)lParam;
	return TreeView_SetItem(hTreeView, &tvit);
}

/*
 * treeview_get_lparam - ACeɊ֘Atꂽ̎擾
 */
LPARAM treeview_get_lparam(const HWND hTreeView, const HTREEITEM hItem)
{
	TV_ITEM tvit;

	if (hItem == NULL) {
		return 0;
	}
	tvit.mask = TVIF_PARAM;
	tvit.hItem = hItem;
	tvit.lParam = 0;
	TreeView_GetItem(hTreeView, &tvit);
	return tvit.lParam;
}

/*
 * treeview_lparam_to_item - LPARAMACe
 */
HTREEITEM treeview_lparam_to_item(const HWND hTreeView, const HTREEITEM hParent, const LPARAM lParam)
{
	HTREEITEM hItem;
	HTREEITEM ret_item;

	if (hParent == NULL) {
		return NULL;
	}
	hItem = TreeView_GetChild(hTreeView, hParent);
	while (hItem != NULL) {
		if (treeview_get_lparam(hTreeView, hItem) == lParam) {
			return hItem;
		}
		if ((ret_item = treeview_lparam_to_item(hTreeView, hItem, lParam)) != NULL) {
			return ret_item;
		}
		hItem = TreeView_GetNextSibling(hTreeView, hItem);
	}
	return NULL;
}

/*
 * treeview_get_rootitem - ACẽ[g擾
 */
HTREEITEM treeview_get_rootitem(const HWND hTreeView, const HTREEITEM hItem)
{
	HTREEITEM ret_item = hItem;
	HTREEITEM wk_item = hItem;

	while ((wk_item = TreeView_GetParent(hTreeView, wk_item)) != NULL) {
		ret_item = wk_item;
	}
	return ret_item;
}

/*
 * treeview_get_hitest - }EX̉̃ACe擾
 */
HTREEITEM treeview_get_hitest(const HWND hTreeView)
{
	TV_HITTESTINFO tvht;
	POINT apos;
	RECT treeview_rect;

	// }EẌʒu擾
	GetCursorPos((LPPOINT)&apos);
	GetWindowRect(hTreeView, (LPRECT)&treeview_rect);
	apos.x = apos.x - treeview_rect.left;
	apos.y = apos.y - treeview_rect.top;

	tvht.pt = apos;
	tvht.flags = TVHT_NOWHERE;
	tvht.hItem = NULL;
	return TreeView_HitTest(hTreeView, &tvht);
}

/*
 * treeview_find_item - LPARAMACe
 */
HTREEITEM treeview_find_item(const HWND hTreeView, const HTREEITEM hParent, const TCHAR *title)
{
	HTREEITEM hItem;
	NODE_INFO *ni;
	TCHAR buf[BUF_SIZE];
	UINT hash;

	if (hParent == NULL) {
		return NULL;
	}
	hash = str2hash(title);
	hItem = TreeView_GetChild(hTreeView, hParent);
	while (hItem != NULL) {
		ni = (NODE_INFO *)treeview_get_lparam(hTreeView, hItem);
		if (ni == NULL || ni->hash == 0 || hash == ni->hash) {
			treeview_get_text(hTreeView, hItem, buf);
			if (lstrcmpi(buf, title) == 0) {
				return hItem;
			}
		}
		hItem = TreeView_GetNextSibling(hTreeView, hItem);
	}
	return NULL;
}

/*
 * treeview_add_node_name - c[r[Ƀm[hǉ
 */
BOOL treeview_add_node_name(const HWND hTreeView, HTREEITEM hRoot, TCHAR *name)
{
	HTREEITEM hParent = hRoot;
	HTREEITEM hItem;
	NODE_INFO *ni;

	// O
	if (*name == TEXT('\0')) {
		return TRUE;
	}
	hItem = treeview_find_item(hTreeView, hParent, name);
	if (hItem != NULL) {
		return TRUE;
	}
	ni = mem_calloc(sizeof(NODE_INFO));
	lstrcpy(ni->name, name);
	ni->hash = str2hash(name);
	hParent = treeview_set_item(hTreeView, name,
		(HTREEITEM)hParent, (HTREEITEM)TVI_SORT, ICON_NAME, ICON_NAME, (LPARAM)ni);
	if (hParent == NULL) {
		return FALSE;
	}
	// N
	ni = mem_calloc(sizeof(NODE_INFO));
	lstrcpy(ni->name, name);
	ni->sub_type = EXTRACT_SUB_TYPE_YEAR;
	treeview_set_item(hTreeView, message_get_res(IDS_STRING_SUBTYPE_YEAR),
		(HTREEITEM)hParent, (HTREEITEM)TVI_LAST, ICON_DATE, ICON_DATE, (LPARAM)ni);
	// 
	ni = mem_calloc(sizeof(NODE_INFO));
	lstrcpy(ni->name, name);
	ni->sub_type = EXTRACT_SUB_TYPE_MONTH;
	treeview_set_item(hTreeView, message_get_res(IDS_STRING_SUBTYPE_MONTH),
		(HTREEITEM)hParent, (HTREEITEM)TVI_LAST, ICON_DATE, ICON_DATE, (LPARAM)ni);
	// 
	ni = mem_calloc(sizeof(NODE_INFO));
	lstrcpy(ni->name, name);
	ni->sub_type = EXTRACT_SUB_TYPE_DAY;
	treeview_set_item(hTreeView, message_get_res(IDS_STRING_SUBTYPE_DAY),
		(HTREEITEM)hParent, (HTREEITEM)TVI_LAST, ICON_DATE, ICON_DATE, (LPARAM)ni);
	// S
	ni = mem_calloc(sizeof(NODE_INFO));
	lstrcpy(ni->name, name);
	ni->sub_type = EXTRACT_SUB_TYPE_ALL;
	treeview_set_item(hTreeView, message_get_res(IDS_STRING_SUBTYPE_ALL),
		(HTREEITEM)hParent, (HTREEITEM)TVI_LAST, ICON_DATA_M, ICON_DATA_M, (LPARAM)ni);
	return TRUE;
}

/*
 * treeview_get_node_name - m[hɊ֘AtꂽO擾
 */
UINT treeview_get_node_name(const HWND hTreeView, const HTREEITEM hItem, TCHAR *name)
{
	NODE_INFO *ni;

	if ((ni = (NODE_INFO *)treeview_get_lparam(hTreeView, hItem)) == NULL) {
		return 0;
	}
	lstrcpy(name, ni->name);
	return ni->hash;
}

/*
 * treeview_add_node_date - c[r[Ƀm[hǉ
 */
BOOL treeview_add_node_date(const HWND hTreeView, HTREEITEM hRoot, const SYSTEMTIME *date)
{
	HTREEITEM hParent = hRoot;
	HTREEITEM hItem;
	NODE_INFO *ni;
	TCHAR buf[BUF_SIZE];

	// N
	wsprintf(buf, message_get_res(IDS_STRING_TREE_YEAR), date->wYear);
	hItem = treeview_find_item(hTreeView, hParent, buf);
	if (hItem == NULL) {
		ni = mem_calloc(sizeof(NODE_INFO));
		ni->date.wYear = date->wYear;
		ni->hash = str2hash(buf);
		hItem = treeview_set_item(hTreeView, buf,
			(HTREEITEM)hParent, (HTREEITEM)TVI_SORT, ICON_DATE, ICON_DATE, (LPARAM)ni);
		if (hItem == NULL) {
			return FALSE;
		}
	}
	hParent = hItem;
	// 
	wsprintf(buf, message_get_res(IDS_STRING_TREE_MONTH), date->wMonth);
	hItem = treeview_find_item(hTreeView, hParent, buf);
	if (hItem == NULL) {
		ni = mem_calloc(sizeof(NODE_INFO));
		ni->date.wYear = date->wYear;
		ni->date.wMonth = date->wMonth;
		ni->hash = str2hash(buf);
		hItem = treeview_set_item(hTreeView, buf,
			(HTREEITEM)hParent, (HTREEITEM)TVI_SORT, ICON_DATE, ICON_DATE, (LPARAM)ni);
		if (hItem == NULL) {
			return FALSE;
		}
	}
	hParent = hItem;
	// 
	wsprintf(buf, message_get_res(IDS_STRING_TREE_DATE), date->wDay);
	hItem = treeview_find_item(hTreeView, hParent, buf);
	if (hItem == NULL) {
		ni = mem_calloc(sizeof(NODE_INFO));
		ni->date.wYear = date->wYear;
		ni->date.wMonth = date->wMonth;
		ni->date.wDay = date->wDay;
		ni->hash = str2hash(buf);
		hItem = treeview_set_item(hTreeView, buf,
			(HTREEITEM)hParent, (HTREEITEM)TVI_SORT, ICON_DATE, ICON_DATE, (LPARAM)ni);
		if (hItem == NULL) {
			return FALSE;
		}
	}
	return TRUE;
}

/*
 * treeview_get_node_date - m[hɊ֘Atꂽt擾
 */
BOOL treeview_get_node_date(const HWND hTreeView, const HTREEITEM hItem, SYSTEMTIME *date)
{
	NODE_INFO *ni;

	if ((ni = (NODE_INFO *)treeview_get_lparam(hTreeView, hItem)) == NULL) {
		return FALSE;
	}
	ZeroMemory(date, sizeof(SYSTEMTIME));
	date->wYear = ni->date.wYear;
	date->wMonth = ni->date.wMonth;
	date->wDay = ni->date.wDay;
	return TRUE;
}

/*
 * treeview_add_node_all - c[r[Ƀm[hǉ
 */
BOOL treeview_add_node_all(const HWND hTreeView, HTREEITEM hRoot)
{
	HTREEITEM hParent;
	NODE_INFO *ni;

	if (hRoot == NULL) {
		hRoot = TreeView_GetRoot(hTreeView);
	}
	hParent = hRoot;
	if (TreeView_GetChild(hTreeView, hParent) != NULL) {
		return FALSE;
	}
	// N
	ni = mem_calloc(sizeof(NODE_INFO));
	ni->sub_type = EXTRACT_SUB_TYPE_YEAR;
	treeview_set_item(hTreeView, message_get_res(IDS_STRING_SUBTYPE_YEAR),
		(HTREEITEM)hParent, (HTREEITEM)TVI_LAST, ICON_DATE, ICON_DATE, (LPARAM)ni);
	// 
	ni = mem_calloc(sizeof(NODE_INFO));
	ni->sub_type = EXTRACT_SUB_TYPE_MONTH;
	treeview_set_item(hTreeView, message_get_res(IDS_STRING_SUBTYPE_MONTH),
		(HTREEITEM)hParent, (HTREEITEM)TVI_LAST, ICON_DATE, ICON_DATE, (LPARAM)ni);
	// 
	ni = mem_calloc(sizeof(NODE_INFO));
	ni->sub_type = EXTRACT_SUB_TYPE_DAY;
	treeview_set_item(hTreeView, message_get_res(IDS_STRING_SUBTYPE_DAY),
		(HTREEITEM)hParent, (HTREEITEM)TVI_LAST, ICON_DATE, ICON_DATE, (LPARAM)ni);
	// S
	ni = mem_calloc(sizeof(NODE_INFO));
	ni->sub_type = EXTRACT_SUB_TYPE_ALL;
	treeview_set_item(hTreeView, message_get_res(IDS_STRING_SUBTYPE_ALL),
		(HTREEITEM)hParent, (HTREEITEM)TVI_LAST, ICON_DATA_M, ICON_DATA_M, (LPARAM)ni);
	return TRUE;
}

/*
 * treeview_get_node_info - IĂm[h̒o擾
 */
BOOL treeview_get_node_info(const HWND hTreeView, const HTREEITEM get_item, EXTRACT_INFO *exi)
{
	NODE_INFO *ni;

	if (get_item == NULL || exi == NULL) {
		return FALSE;
	}
	ni = (NODE_INFO *)treeview_get_lparam(hTreeView, get_item);
	if (ni == NULL) {
		// 
		exi->sub_type = EXTRACT_SUB_TYPE_NONE;
		*exi->name = TEXT('\0');
		exi->name_hash = 0;
		ZeroMemory(&exi->date, sizeof(SYSTEMTIME));
		return TRUE;
	}
	exi->sub_type = ni->sub_type;
	lstrcpy(exi->name, ni->name);
	exi->name_hash = str2hash(ni->name);
	exi->date = ni->date;
	return TRUE;
}

/*
 * treeview_free_node_info - m[h̉
 */
void treeview_free_node_info(const HWND hTreeView, const HTREEITEM hParent)
{
	HTREEITEM hItem;
	NODE_INFO *ni;

	if (hParent == NULL) {
		return;
	}
	// 
	ni = (NODE_INFO *)treeview_get_lparam(hTreeView, hParent);
	if (ni != NULL) {
		treeview_set_lparam(hTreeView, hParent, 0);
		mem_free(&ni);
	}
	// qm[h
	hItem = TreeView_GetChild(hTreeView, hParent);
	while (hItem != NULL) {
		treeview_free_node_info(hTreeView, hItem);
		hItem = TreeView_GetNextSibling(hTreeView, hItem);
	}
}
/* End of source */
